/*
 * phc -- the open source PHP compiler
 * See license/README.license for licensing information
 *
 * Error handling 
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "error.h"

// printf format statements are supported

static char* error_msg[] = {
	NULL, /* start at error code 1 */
	/* ERR_BISON */ "%s",
	/* ERR_EXPECT_VAR */ "Variable name expected",
	/* ERR_FOREACH_KEY_REF */ "Key element cannot be a reference",
	/* ERR_ABSTRACT_VARS */ "Cannot declare variables to be abstract",
	/* ERR_FINAL_VARS */ "The final modifier is only allowed for methods",
	/* ERR_XML_PARSE */ "Could not parse the XML",
	/* ERR_INCLUDE_CLASS_EXISTS */ "Included class %s at %s:%d already exists",
	/* ERR_INCLUDE_METHOD_EXISTS */ "Included method %s at %s:%d already exists",
	/* ERR_INCLUDE_BAD_PARSE */ "Included file %s does not parse",
	/* ERR_INCLUDE_NO_FILE*/ "Included file %s could not be opened",
	/* ERR_INCLUDE_RETURN */ "Included file uses return statement, which is unsupported, at %s:%d",
};

static char* warning_msg[] = {
	NULL, /* start at error code 1 */
};

static char* internal_error_msg[] = {
	/* INTERNAL_ERR_LOOPING_TRANSFORM */ "Transform `%s' appears to be looping on a node of type `%s'",
};

// Print the error message and quit
void error(int code, int line, ...)
{
	va_list argp;
	
	fprintf(stderr, "Error: ");

	va_start(argp, line);
	vfprintf(stderr, error_msg[code], argp);
	va_end(argp);
	
	if(line > 0)
		fprintf(stderr, ", around line %d", line);
	
	fprintf(stderr, "\n");
	exit(code);
}

// Print the error message and quit
void internal_error(int code, ...)
{
	va_list argp;
	
	fprintf(stderr, "Internal error: ");

	va_start(argp, code);
	vfprintf(stderr, internal_error_msg[code], argp);
	va_end(argp);
	
	fprintf(stderr, "\n");
	exit(ERR_INTERNAL);
}

// Print a warning and continue
void warning(int code, int line, ...)
{
	va_list argp;
	
	fprintf(stderr, "Warning: ");

	va_start(argp, line);
	vfprintf(stderr, warning_msg[code], argp);
	va_end(argp);
	
	if(line > 0)
		fprintf(stderr, ", around line %d", line);
	
	fprintf(stderr, "\n");
}
