/*
 * phc -- the open source PHP compiler
 * See license/README.license for licensing information
 *
 * Lexical analyser for PHP. PHPLexer::yylex is generated by flex.
 */

#ifndef PHC_PHP_LEXER
#define PHC_PHP_LEXER

#ifdef YYSTYPE
#undef YYSTYPE
#endif
#define YYSTYPE Object*

#include <FlexLexer.h>
#include "lib/String.h"
#include "lib/List.h"
#include "Lexer.h"
#include "ast.h"

class PHP_lexer : public yyFlexLexer, public Lexer
{
private:
	YYSTYPE semantic_value;
	istream* is;

public: // Required by Lexer
	int yylex(); // Body provided by flex

	YYSTYPE get_yylval()
	{
		return semantic_value;
	}

public:
	PHP_lexer(istream* _is)
	{
		is = _is;
		yyin = _is;
		source_line = 1;
		attach_to_previous = false;
		source_column = 0;
		mt_index = 0;
		mt_count = 0;
		last_commented_node = 0;
		after_arrow = false;
	}
	
	void return_to_complex_syntax(); // Body defined in php.lex

	void attach_comment(String *s)
	{
		assert(last_commented_node);
		last_commented_node->comments->push_back(s);
	}	

public: // Additional state
	/*
	 * Interfacing with bison
	 */

	long source_line;
	List<String*> last_comments;
	AST_commented_node* last_commented_node;

	/*
	 * Process comments and HEREDOC strings 
	 */

	bool attach_to_previous; // Attach comment to previous token?


	char *heredoc_id, *heredoc_id_ptr;
	long heredoc_id_len;

	/*
	 * buffer manipulation
	 */

	string buffer;

	/*
	 * Returning multiple tokens from a single production
	 */

	#define MAX_MT 10

	long mt_type[MAX_MT];
	String* mt_lval[MAX_MT];
	long mt_index;
	long mt_count;
	long mt_final_state;

	// Schedule the return of a token of type "type", with semantic value
	// "lval". If "lval" is nonnull, "length" must be the length of "lval",
	// or -1 to return the entire string. lval must be zero-terminated. 

	void schedule_return(long type, const char* lval = 0, long length = -1)
	{
		mt_type[mt_count] = type;

		if(lval)
		{
			if(length == -1)
				mt_lval[mt_count] = new String(lval);
			else
				mt_lval[mt_count] = new String(lval, length);
		}
		else
		{
			mt_lval[mt_count] = NULL;
		}

		mt_count++;
	}

	void schedule_return(long type, string& s)
	{
		schedule_return(type, s.c_str(), s.size());
	}

	long return_state;

private:
	long source_column;	// source_column is inaccurate and should
											// be used within the lexer only
	
	bool after_arrow; // was the last token we generated ->?
};

#endif
