#include "PHP_parser.h"
#include "Invocation_targets.h"
#include "Variable_targets.h"
#include "Remove_concat_null.h"
#include "Token_conversion.h"
#include "Process_includes.h"
#include "cmdline.h"

bool 
PHP_parser::parse()
{
	if (yyparse(this) == 0)
	{
		Invocation_targets it;
		php_script->visit(&it);	

		Variable_targets vt;
		php_script->visit(&vt);

		// The parser will introduce concatenations with "" in certain
		// (frequent) circumstances. This will remove them. This could
		// probably be run at any time.
		Remove_concat_null rcn;
		php_script->transform(&rcn);

		// There are a number of strange rules based on the value of an
		// integer/real, which must be applied. The parser cannot do this,
		// since we have to determine whether there are 1 or more - signs
		// before the number, and this is unknown at compile time. This
		// should be run early.
		Token_conversion tc;
		php_script->transform(&tc);

		// This replaces include() statements with the file they include,
		// based on certain rules. Since the included script will be
		// pre-parsed, process_includes should be one of the last transforms
		// to run, else the effort will be duplicated (which could lead to
		// an error)
		if(args_info.compile_time_includes_flag)
		{
			Process_includes pi;
			php_script->transform(&pi);
		}

		return true;
	}
	else
	{
		return false;
	}

}
