/*
 * phc -- the open source php compiler
 * See license/README.license for licensing information
 *
 * Parser context for the PHP parser 
 */

#ifndef PHC_PHP_PARSER
#define PHC_PHP_PARSER

#include "Parser_context.h"
#include "PHP_lexer.h"
#include "lib/error.h"

int yyparse(void*);

class PHP_parser : public Parser_context
{
public: 
	// We keep our own (typed) copy of the lexer
	PHP_lexer* php_lexer;

	// A global filename is not sufficient since we may be including files
	String* filename;


public:
	PHP_parser(istream* file, String* fn) : Parser_context(php_lexer = new PHP_lexer(file))
	{
		php_script = new AST_php_script();
		current_method = new String("[outside any method]");
		current_class = new String("[outside any class]");
		filename = fn;
	}

/*	PHP_parser(PHP_lexer* lexer) : Parser_context(lexer)
	{
		php_lexer = lexer;
		php_script = new AST_php_script();
		current_method = new String("[outside any method]");
		current_class = new String("[outside any class]");
	}
*/
public: // Additional state
	AST_php_script* php_script;
	String* current_method;
	String* current_class;

public:

	// defining it in the header file led to problems, so this is defined
	// in PHP_parser.cpp
	bool parse();
	
	void yyerror(const char* s)
	{
		error(ERR_BISON, php_lexer->source_line, s);
	}
};

#endif
